﻿<?PHP
	## Configuration #######################################################################

	$root = dirname(dirname(__DIR__));
	define( 'AUTO_DOMAIN_ROOT', $root.'/domains' );
	define( 'AUTO_DOMAIN_OPENSERV_CONF', $root.'/userdata/profiles/Default_domains.txt' );

	## Directly script #####################################################################

	class domainLib2
	{
		/**
		 * Название файла который по умолчанию
         * может находится в папке домена
         * и прописывать локальные настройки
		 */
		const NOAUTODOMAIN = '.noautodomain';
		/**
		 * Основная директория домена
		 */
		const DEFHOME = 'www';
		/**
		 * Стандартный файл настроек доменов openserver
		 */
		const OPENSERV_CONF = AUTO_DOMAIN_OPENSERV_CONF;
		/**
		 * Директория сканирования
		 * 
		 * @var string
		 */
		protected $_root_dir = AUTO_DOMAIN_ROOT;
		/**
		 * Ассоциативный массив основных доменов и их поддоменов
		 * 
		 * @var array
		 */
		protected $_data_tree = null;
		/**
		 * Массив всех доменов
		 * 
		 * @var array
		 */
		protected $_data_list = null;
		/**
		 * Инициализация класса
		 * 
		 * @param  void
		 * @return void
		 * @access public
		 */
		public function __construct()
		{
			$this->_makeDomainData();
		}
		/**
		 * Сканирует директории и подготавливает список доменов и их соотношений
		 * 
		 * @param  void
		 * @return int
		 * @access protected
		 */
		protected function _makeDomainData()
		{
			$root = $this->_root_dir;
			$home = $this::DEFHOME;
            $map_file = $this::NOAUTODOMAIN;
			//
			$list1 = glob( $root.'/*', GLOB_ONLYDIR );
			//
			$domain_tree = array();
			$domain_list = array();
			foreach( $list1 as $first_folder )
			{
				$first_name = basename($first_folder);
				$domain_tree[$first_name] = $this->scanDomainFolder( $root, $home, $map_file, $first_folder );
			}
            //
            $array_iterator = new RecursiveIteratorIterator(
                new RecursiveArrayIterator($domain_tree)
            );
            foreach($array_iterator as $key=>$value){
                $keys = array();
                // in this case i skip the grand parent (numeric array)
                for ($i = $array_iterator->getDepth()-1; $i>=0; $i--){
                    $keys[] = $array_iterator->getSubIterator($i)->key();
                }
                $domain_list[implode('.',$keys)] = $value;
            }
            //
            //
			$this->_data_tree = $domain_tree;
			$this->_data_list = $domain_list;
			//
			return count($domain_list);
		}
		/**
		 * Сканирует директории и подготавливает дерево
         * локально найденных доменов и их настроек.
         * 
         * Используется рекурсия.
         * 
		 * @param  string  link root directory
         * @param  string  name default home folder
         * @param  string  name of map file
		 * @param  string  link scan directory
         * @return array   domain tree list
		 * @access protected
		 */
		protected function scanDomainFolder( $root, $home, $map_file, $first_folder )
		{
            $home_domain = str_replace( $root, null, $first_folder );
			$domain_tree = array(
                $home => null,
            );
			// individual map
			if( file_exists($first_folder.'/'.$map_file) )
            {
                $domain_tree = $this->loadNAutoDFile( $home_domain, $first_folder.'/'.$map_file );
                //
                foreach( $domain_tree as $d_key => &$d_folder )
                {
                    if( $d_key == $home )continue;
                    $d_folder = array( $home=>$d_folder );
                }
                unset( $d_folder );
                if( empty($domain_tree[$home]) )$domain_tree[$home] = $home_domain;
                //
				return $domain_tree;
			}
			// is root domain only
			if( !is_dir($first_folder.'/'.$home) ){
                $domain_tree[$home] = $home_domain;
				return $domain_tree;
			}
			//
			// standart denwer map+
            $list2 = glob( $first_folder.'/*', GLOB_ONLYDIR );
            foreach( $list2 as $sub_domain_folder ){
                $sub_home_domain = str_replace( $root, null, $sub_domain_folder );
				$sub_domain_name = basename($sub_domain_folder);
                if( $sub_domain_name == $home ){
                    $domain_tree[$sub_domain_name] = $sub_home_domain;
                }else{
				    $domain_tree[$sub_domain_name] = $this->scanDomainFolder( $root, $home, $map_file, $sub_domain_folder );
                }
            }
            return $domain_tree;
		}
		/**
		 * Загружает файл настройки домена
         * 
         * Формат файла ini
         * 
         * Пример файла:
         * www = 'public_web_folder' 
         * domain2 = 'home_folder_subdomain/public_web_folder'
         * ...
		 * 
		 * @param  string  domain directory
         * @param  string  link to map file
		 * @return array  local domain map data
		 * @access protected
		 */
		protected function loadNAutoDFile( $domain_folder, $map_file )
		{
            $d_list = array();
            //
            if( file_exists($map_file) && filesize($map_file) ){
                $ini = @parse_ini_file($map_file);
                if( !$ini )return $d_list;
                //
				foreach( $ini as $one_domain => $subfolder ){
				    $d_list[$one_domain] = $domain_folder.'/'.$subfolder;
				}
            }
			return $d_list;
		}
		/**
		 * Возвращает массив доменов
		 * 
		 * @param  void
		 * @return array
		 * @access private
		 */
		public function getDomainList()
		{
			return $this->_data_list;
		}
		/**
		 * Возвращает ассоциативный массив основных доменов и их поддоменов
		 * 
		 * @param  void
		 * @return array
		 * @access private
		 */
		public function getDomainTree()
		{
			return $this->_data_tree;
		}
		/**
         * Печатает либо возвращает html со списком доменов         * 
         * 
         * @param [int count column]
         * @param [bool  return html]
         * @return mixed  html data or count domains
		 * @access public
		 */
		public function echoDomainList( $count_column = 4, $only_return = false )
		{
        	$list = $this->getDomainTree();
            //
            $array_iterator = new RecursiveIteratorIterator(
                new RecursiveArrayIterator($list)
            );
            $n = 0;
            foreach($array_iterator as $folder){ $n++; }
            $count_domains = $n;
            //
            $n = 0;
            $last_depth = 1;
            $count_one_column = ceil( $count_domains / $count_column );
            $html = null;
            //
            $html .= (str_repeat("\t", 0)."<ul>\n");
            foreach($array_iterator as $key=>$folder)
            {
                $n++;
                $depth = $array_iterator->getDepth();
                //
                // monitoring opened and closed group
                if( $last_depth != $depth ){
                    if( $last_depth > $depth ){
                        for( $j = $depth;$j<$last_depth;$j++ ){
                            $html .= (str_repeat("\t", $depth)."</ul>\n");
                        }
                    }else{
                        for( $j = $last_depth;$j<$depth;$j++ ){
                            $html .= (str_repeat("\t", $last_depth)."<ul>\n");
                        }
                    }
                    $last_depth = $depth;
                }
                //
                // in this case i skip the grand parent (numeric array)
                $keys = array();
                for ($i = $depth-1; $i>=0; $i--){
                    $keys[] = $array_iterator->getSubIterator($i)->key();
                }
                $domain = implode('.',$keys);
                //
                // to next column
                if( $depth == 1 && $n > $count_one_column ){
                    $n = 0;
                    $html .= (str_repeat("\t", 0)."</ul>\n");
                    $html .= (str_repeat("\t", 0)."<ul>\n");
                }
                //
                // item
                $html .= (str_repeat("\t", $last_depth)."<li><a href='//$domain/' title='FOLDER: $folder'>$domain</a></li>\n");
            }
            // close all group
            for( $j = $last_depth-1;$j>=0;$j-- ){
                $html .= (str_repeat("\t", $j)."</ul>\n");
            }
            //
            if( $only_return ){
                return $html;
            }
            echo( $html );
            return $count_domains;
		}
		/**
		 * Записывает стандартный файл настроек доменов openserver
		 * 
		 * @param  void
		 * @return string  json
		 * @access private
		 */
		public function writeDomainList()
		{
			if( !file_exists($this::OPENSERV_CONF) )return false;
			//
			$list = $this->getDomainList();
			//
			$data = null;
			foreach( $list as $domain =>$folder ){
				$data .= $domain.';'.str_replace( '\\', '/', $folder )."\n";
			}
			$size = file_put_contents( $this::OPENSERV_CONF, $data );
			//
			return '{"domain":'.count($list).',"write":'.$size.'}';
		}
	}

	## End of file #########################################################################

// EOF